@@ -23,6 +23,9 @@ class FileLocation(Enum):
     S3 = "s3"  # Amazon S3
     # [END filelocation]
 
+    def __str__(self) -> str:
+        return self.value
+
 
 class FileType(Enum):
     # [START filetypes]
@@ -32,6 +35,9 @@ class FileType(Enum):
     PARQUET = "parquet"
     # [END filetypes]
 
+    def __str__(self) -> str:
+        return self.value
+
 
 class Database(Enum):
     # [START database]
@@ -40,8 +46,12 @@ class Database(Enum):
     SQLITE = "sqlite"
     BIGQUERY = "bigquery"
     SNOWFLAKE = "snowflake"
+    REDSHIFT = "redshift"
     # [END database]
 
+    def __str__(self) -> str:
+        return self.value
+
 
 SUPPORTED_FILE_LOCATIONS = [const.value for const in FileLocation]
 SUPPORTED_FILE_TYPES = [const.value for const in FileType]
