@@ -1,14 +1,13 @@
 import importlib
 from pathlib import Path
+from typing import TYPE_CHECKING
 
-from airflow.hooks.base import BaseHook
-
-from astro.databases.base import BaseDatabase
 from astro.utils.path import get_class_name, get_dict_with_module_names_to_dot_notations
 
-DEFAULT_CONN_TYPE_TO_MODULE_PATH = get_dict_with_module_names_to_dot_notations(
-    Path(__file__)
-)
+if TYPE_CHECKING:
+    from astro.databases.base import BaseDatabase
+
+DEFAULT_CONN_TYPE_TO_MODULE_PATH = get_dict_with_module_names_to_dot_notations(Path(__file__))
 CUSTOM_CONN_TYPE_TO_MODULE_PATH = {
     "gcpbigquery": DEFAULT_CONN_TYPE_TO_MODULE_PATH["bigquery"],
     "google_cloud_platform": DEFAULT_CONN_TYPE_TO_MODULE_PATH["bigquery"],
@@ -20,15 +19,17 @@ CONN_TYPE_TO_MODULE_PATH = {
 SUPPORTED_DATABASES = set(DEFAULT_CONN_TYPE_TO_MODULE_PATH.keys())
 
 
-def create_database(conn_id: str) -> BaseDatabase:
+def create_database(conn_id: str) -> "BaseDatabase":
     """
     Given a conn_id, return the associated Database class.
 
     :param conn_id: Database connection ID in Airflow
     """
+    from airflow.hooks.base import BaseHook
+
     conn_type = BaseHook.get_connection(conn_id).conn_type
     module_path = CONN_TYPE_TO_MODULE_PATH[conn_type]
     module = importlib.import_module(module_path)
     class_name = get_class_name(module_ref=module, suffix="Database")
-    database: BaseDatabase = getattr(module, class_name)(conn_id)
+    database: "BaseDatabase" = getattr(module, class_name)(conn_id)
     return database
