@@ -6,24 +6,10 @@ import os
 import random
 import string
 from dataclasses import dataclass, field
-from typing import Any
+from typing import Any, Sequence
 
 import pandas as pd
 from airflow.providers.snowflake.hooks.snowflake import SnowflakeHook
-from astro import settings
-from astro.constants import (
-    DEFAULT_CHUNK_SIZE,
-    ColumnCapitalization,
-    FileLocation,
-    FileType,
-    LoadExistStrategy,
-    MergeConflictStrategy,
-)
-from astro.databases.base import BaseDatabase
-from astro.exceptions import DatabaseCustomError
-from astro.files import File
-from astro.settings import SNOWFLAKE_SCHEMA
-from astro.sql.table import BaseTable, Metadata
 from snowflake.connector import pandas_tools
 from snowflake.connector.errors import (
     DatabaseError,
@@ -37,6 +23,23 @@ from snowflake.connector.errors import (
     RequestTimeoutError,
     ServiceUnavailableError,
 )
+from sqlalchemy import Column, column, insert, select
+from sqlalchemy.types import VARCHAR
+
+from astro import settings
+from astro.constants import (
+    DEFAULT_CHUNK_SIZE,
+    ColumnCapitalization,
+    FileLocation,
+    FileType,
+    LoadExistStrategy,
+    MergeConflictStrategy,
+)
+from astro.databases.base import BaseDatabase
+from astro.exceptions import DatabaseCustomError
+from astro.files import File
+from astro.settings import LOAD_TABLE_AUTODETECT_ROWS_COUNT, SNOWFLAKE_SCHEMA
+from astro.table import BaseTable, Metadata
 
 DEFAULT_CONN_ID = SnowflakeHook.default_conn_name
 
@@ -99,9 +102,7 @@ class SnowflakeFileFormat:
         return (
             "file_format_"
             + random.choice(string.ascii_lowercase)
-            + "".join(
-                random.choice(string.ascii_lowercase + string.digits) for _ in range(7)
-            )
+            + "".join(random.choice(string.ascii_lowercase + string.digits) for _ in range(7))
         )
 
     def set_file_type_from_file(self, file: File) -> None:
@@ -172,9 +173,7 @@ class SnowflakeStage:
         return (
             "stage_"
             + random.choice(string.ascii_lowercase)
-            + "".join(
-                random.choice(string.ascii_lowercase + string.digits) for _ in range(7)
-            )
+            + "".join(random.choice(string.ascii_lowercase + string.digits) for _ in range(7))
         )
 
     def set_url_from_file(self, file: File) -> None:
@@ -322,9 +321,7 @@ class SnowflakeDatabase(BaseDatabase):
     # ---------------------------------------------------------
 
     @staticmethod
-    def _create_stage_auth_sub_statement(
-        file: File, storage_integration: str | None = None
-    ) -> str:
+    def _create_stage_auth_sub_statement(file: File, storage_integration: str | None = None) -> str:
         """
         Create authentication-related line for the Snowflake CREATE STAGE.
         Raise an exception if it is not defined.
@@ -380,9 +377,7 @@ class SnowflakeDatabase(BaseDatabase):
             `Snowflake official documentation on stage creation
             <https://docs.snowflake.com/en/sql-reference/sql/create-stage.html>`_
         """
-        auth = self._create_stage_auth_sub_statement(
-            file=file, storage_integration=storage_integration
-        )
+        auth = self._create_stage_auth_sub_statement(file=file, storage_integration=storage_integration)
 
         metadata = metadata or self.default_metadata
         stage = SnowflakeStage(metadata=metadata)
@@ -415,9 +410,7 @@ class SnowflakeDatabase(BaseDatabase):
         try:
             self.hook.run(sql_statement)
         except ProgrammingError:
-            logging.error(
-                "Stage '%s' does not exist or not authorized.", stage.qualified_name
-            )
+            logging.error("Stage '%s' does not exist or not authorized.", stage.qualified_name)
             return False
         return True
 
@@ -443,12 +436,9 @@ class SnowflakeDatabase(BaseDatabase):
         :param file: File used to check the file type of to decide
             whether there is a native auto detection available for it.
         """
-        is_file_type_supported = (
-            file.type.name in NATIVE_AUTODETECT_SCHEMA_SUPPORTED_FILE_TYPES
-        )
+        is_file_type_supported = file.type.name in NATIVE_AUTODETECT_SCHEMA_SUPPORTED_FILE_TYPES
         is_file_location_supported = (
-            file.location.location_type
-            in NATIVE_AUTODETECT_SCHEMA_SUPPORTED_FILE_LOCATIONS
+            file.location.location_type in NATIVE_AUTODETECT_SCHEMA_SUPPORTED_FILE_LOCATIONS
         )
         return is_file_type_supported and is_file_location_supported
 
@@ -487,48 +477,79 @@ class SnowflakeDatabase(BaseDatabase):
             },
         )
 
+    @classmethod
+    def use_quotes(cls, cols: Sequence[str]) -> bool:
+        """
+        With snowflake identifier we have two cases,
+
+        1. When Upper/Mixed case col names are used
+            We are required to preserver the text casing of the col names. By adding the quotes around identifier.
+        2. When lower case col names are used
+            We can use them as is
+
+        This is done to be in sync with Snowflake SQLAlchemy dialect.
+        https://docs.snowflake.com/en/user-guide/sqlalchemy.html#object-name-case-handling
+
+        Snowflake stores all case-insensitive object names in uppercase text. In contrast, SQLAlchemy considers all
+        lowercase object names to be case-insensitive. Snowflake SQLAlchemy converts the object name case during
+        schema-level communication (i.e. during table and index reflection). If you use uppercase object names,
+        SQLAlchemy assumes they are case-sensitive and encloses the names with quotes. This behavior will cause
+        mismatches against data dictionary data received from Snowflake, so unless identifier names have been truly
+        created as case sensitive using quotes (e.g. "TestDb"), all lowercase names should be used on the SQLAlchemy
+        side.
+
+        :param cols: list of columns
+        """
+        return any(col for col in cols if not col.islower() and not col.isupper())
+
     def create_table_using_schema_autodetection(
         self,
         table: BaseTable,
         file: File | None = None,
         dataframe: pd.DataFrame | None = None,
-        columns_names_capitalization: ColumnCapitalization = "lower",
+        columns_names_capitalization: ColumnCapitalization = "original",
     ) -> None:
         """
         Create a SQL table, automatically inferring the schema using the given file.
+        Overriding default behaviour and not using the `prep_table` since it doesn't allow the adding quotes.
 
         :param table: The table to be created.
         :param file: File used to infer the new table columns.
         :param dataframe: Dataframe used to infer the new table columns if there is no file
         """
+        if file is None:
+            if dataframe is None:
+                raise ValueError(
+                    "File or Dataframe is required for creating table using schema autodetection"
+                )
+            source_dataframe = dataframe
+        else:
+            source_dataframe = file.export_to_dataframe(nrows=LOAD_TABLE_AUTODETECT_ROWS_COUNT)
 
-        # Snowflake don't expect mixed case col names like - 'Title' or 'Category'
-        # we explicitly convert them to lower case, if not provided by user
-        if columns_names_capitalization not in ["lower", "upper"]:
-            columns_names_capitalization = "lower"
+        # We are changing the case of table name to ease out on the requirements to add quotes in raw queries.
+        # ToDO - Currently, we cannot to append using load_file to a table name which is having name in lower case.
+        pandas_tools.write_pandas(
+            conn=self.hook.get_conn(),
+            df=source_dataframe,
+            table_name=table.name.upper(),
+            schema=table.metadata.schema,
+            database=table.metadata.database,
+            chunk_size=DEFAULT_CHUNK_SIZE,
+            quote_identifiers=self.use_quotes(source_dataframe),
+            auto_create_table=True,
+        )
+        # We are truncating since we only expect table to be created with required schema.
+        # Since this method is used by both native and pandas path we cannot skip this step.
+        self.truncate_table(table)
 
-        if file:
-            dataframe = file.export_to_dataframe(
-                nrows=settings.LOAD_TABLE_AUTODETECT_ROWS_COUNT,
-                columns_names_capitalization=columns_names_capitalization,
-            )
-
-        # Snowflake doesn't handle well mixed capitalisation of column name chars
-        # we are handling this more gracefully in a separate PR
-        super().create_table_using_schema_autodetection(table, dataframe=dataframe)
-
-    def is_native_load_file_available(
-        self, source_file: File, target_table: BaseTable
-    ) -> bool:
+    def is_native_load_file_available(self, source_file: File, target_table: BaseTable) -> bool:
         """
         Check if there is an optimised path for source to destination.
 
         :param source_file: File from which we need to transfer data
         :param target_table: Table that needs to be populated with file data
         """
-        is_file_type_supported = (
-            source_file.type.name in NATIVE_LOAD_SUPPORTED_FILE_TYPES
-        )
+        is_file_type_supported = source_file.type.name in NATIVE_LOAD_SUPPORTED_FILE_TYPES
         is_file_location_supported = (
             source_file.location.location_type in NATIVE_LOAD_SUPPORTED_FILE_LOCATIONS
         )
@@ -568,15 +589,11 @@ class SnowflakeDatabase(BaseDatabase):
         """
         native_support_kwargs = native_support_kwargs or {}
         storage_integration = native_support_kwargs.get("storage_integration")
-        stage = self.create_stage(
-            file=source_file, storage_integration=storage_integration
-        )
+        stage = self.create_stage(file=source_file, storage_integration=storage_integration)
 
         table_name = self.get_table_qualified_name(target_table)
         file_path = os.path.basename(source_file.path) or ""
-        sql_statement = (
-            f"COPY INTO {table_name} FROM @{stage.qualified_name}/{file_path}"
-        )
+        sql_statement = f"COPY INTO {table_name} FROM @{stage.qualified_name}/{file_path}"
         try:
             self.hook.run(sql_statement)
         except (ValueError, AttributeError) as exe:
@@ -599,17 +616,22 @@ class SnowflakeDatabase(BaseDatabase):
         :param if_exists: Strategy to be used in case the target table already exists.
         :param chunk_size: Specify the number of rows in each batch to be written at a time.
         """
-        if if_exists == "replace":
-            self.create_table(target_table, dataframe=source_dataframe)
 
+        auto_create_table = False
+        if if_exists == "replace" or not self.table_exists(target_table):
+            auto_create_table = True
+
+        # We are changing the case of table name to ease out on the requirements to add quotes in raw queries.
+        # ToDO - Currently, we cannot to append using load_file to a table name which is having name in lower case.
         pandas_tools.write_pandas(
             conn=self.hook.get_conn(),
             df=source_dataframe,
-            table_name=target_table.name,
+            table_name=target_table.name.upper(),
             schema=target_table.metadata.schema,
             database=target_table.metadata.database,
             chunk_size=chunk_size,
-            quote_identifiers=False,
+            quote_identifiers=self.use_quotes(source_dataframe),
+            auto_create_table=auto_create_table,
         )
 
     def get_sqlalchemy_template_table_identifier_and_parameter(
@@ -722,31 +744,36 @@ class SnowflakeDatabase(BaseDatabase):
         source_cols = source_to_target_columns_map.keys()
         target_cols = source_to_target_columns_map.values()
 
+        target_identifier_enclosure = ""
+        if self.use_quotes(list(target_cols)):
+            target_identifier_enclosure = '"'
+
+        source_identifier_enclosure = ""
+        if self.use_quotes(list(source_cols)):
+            source_identifier_enclosure = '"'
+
         (
             source_table_identifier,
             source_table_param,
-        ) = self.get_sqlalchemy_template_table_identifier_and_parameter(
-            source_table, "source_table"
-        )
+        ) = self.get_sqlalchemy_template_table_identifier_and_parameter(source_table, "source_table")
 
         (
             target_table_identifier,
             target_table_param,
-        ) = self.get_sqlalchemy_template_table_identifier_and_parameter(
-            target_table, "target_table"
-        )
+        ) = self.get_sqlalchemy_template_table_identifier_and_parameter(target_table, "target_table")
 
         statement = (
-            f"merge into {target_table_identifier} using {source_table_identifier} "
-            + "on {merge_clauses}"
+            f"merge into {target_table_identifier} using {source_table_identifier} " + "on {merge_clauses}"
         )
 
         merge_target_dict = {
-            f"merge_clause_target_{i}": f"{target_table_name}.{x}"
+            f"merge_clause_target_{i}": f"{target_table_name}."
+            f"{target_identifier_enclosure}{x}{target_identifier_enclosure}"
             for i, x in enumerate(target_conflict_columns)
         }
         merge_source_dict = {
-            f"merge_clause_source_{i}": f"{source_table_name}.{x}"
+            f"merge_clause_source_{i}": f"{source_table_name}."
+            f"{source_identifier_enclosure}{x}{source_identifier_enclosure}"
             for i, x in enumerate(target_conflict_columns)
         }
         statement = statement.replace(
@@ -769,21 +796,26 @@ class SnowflakeDatabase(BaseDatabase):
             statement += " when matched then UPDATE SET {merge_vals}"
             merge_statement = ",".join(
                 [
-                    f"{target_table_name}.{t}={source_table_name}.{s}"
+                    f"{target_table_name}.{target_identifier_enclosure}{t}{target_identifier_enclosure}="
+                    f"{source_table_name}.{source_identifier_enclosure}{s}{source_identifier_enclosure}"
                     for s, t in source_to_target_columns_map.items()
                 ]
             )
             statement = statement.replace("{merge_vals}", merge_statement)
-        statement += (
-            " when not matched then insert({target_columns}) values ({append_columns})"
-        )
+        statement += " when not matched then insert({target_columns}) values ({append_columns})"
         statement = statement.replace(
             "{target_columns}",
-            ",".join(f"{target_table_name}.{t}" for t in target_cols),
+            ",".join(
+                f"{target_table_name}.{target_identifier_enclosure}{t}{target_identifier_enclosure}"
+                for t in target_cols
+            ),
         )
         statement = statement.replace(
             "{append_columns}",
-            ",".join(f"{source_table_name}.{s}" for s in source_cols),
+            ",".join(
+                f"{source_table_name}.{source_identifier_enclosure}{s}{source_identifier_enclosure}"
+                for s in source_cols
+            ),
         )
         params = {
             **merge_target_dict,
@@ -793,6 +825,92 @@ class SnowflakeDatabase(BaseDatabase):
         }
         return statement, params
 
+    def append_table(
+        self,
+        source_table: BaseTable,
+        target_table: BaseTable,
+        source_to_target_columns_map: dict[str, str],
+    ) -> None:
+        """
+        Append the source table rows into a destination table.
+
+        Overriding the base method since we need to add quotes around the identifiers for
+         snowflake to preserve case of cols - Column(name=col, quote=True)
+
+        :param source_table: Contains the rows to be appended to the target_table
+        :param target_table: Contains the destination table in which the rows will be appended
+        :param source_to_target_columns_map: Dict of source_table columns names to target_table columns names
+        """
+        target_table_sqla = self.get_sqla_table(target_table)
+        source_table_sqla = self.get_sqla_table(source_table)
+        use_quotes_target_table = self.use_quotes(target_table_sqla.columns.keys())
+        use_quotes_source_table = self.use_quotes(source_table_sqla.columns.keys())
+        target_columns: list[column]
+        source_columns: list[column]
+
+        if not source_to_target_columns_map:
+            target_columns = [
+                Column(name=col.name, quote=use_quotes_target_table, type_=col.type)
+                for col in target_table_sqla.c.values()
+            ]
+            source_columns = target_columns
+        else:
+            # We are adding the VARCHAR in Column(name=col, quote=True, type_=VARCHAR) as a placeholder since the
+            # Column object requires it. It has no effect on the final query generated.
+            target_columns = [
+                Column(name=col, quote=use_quotes_target_table, type_=VARCHAR)
+                for col in source_to_target_columns_map.keys()
+            ]
+            source_columns = [
+                Column(name=col, quote=use_quotes_source_table, type_=VARCHAR)
+                for col in source_to_target_columns_map.keys()
+            ]
+
+        sel = select(source_columns).select_from(source_table_sqla)
+        # TODO: We should fix the following Type Error
+        # incompatible type List[ColumnClause[<nothing>]]; expected List[Column[Any]]
+        sql = insert(target_table_sqla).from_select(target_columns, sel)  # type: ignore[arg-type]
+        self.run_sql(sql=sql)
+
+    @classmethod
+    def get_merge_initialization_query(cls, parameters: tuple) -> str:
+        """
+        Handles database-specific logic to handle constraints, keeping
+        it agnostic to database.
+        """
+        identifier_enclosure = ""
+        if cls.use_quotes(parameters):
+            identifier_enclosure = '"'
+
+        constraints = ",".join([f"{identifier_enclosure}{p}{identifier_enclosure}" for p in parameters])
+        sql = "ALTER TABLE {{table}} ADD CONSTRAINT airflow UNIQUE (%s)" % constraints
+        return sql
+
+    def openlineage_dataset_name(self, table: BaseTable) -> str:
+        """
+        Returns the open lineage dataset name as per
+        https://github.com/OpenLineage/OpenLineage/blob/main/spec/Naming.md
+        Example: db_name.schema_name.table_name
+        """
+        conn = self.hook.get_connection(self.conn_id)
+        conn_extra = conn.extra_dejson
+        schema = conn_extra.get("schema") or conn.schema
+        db = conn_extra.get("database")
+        return f"{db}.{schema}.{table.name}"
+
+    def openlineage_dataset_namespace(self) -> str:
+        """
+        Returns the open lineage dataset namespace as per
+        https://github.com/OpenLineage/OpenLineage/blob/main/spec/Naming.md
+        Example: snowflake://ACCOUNT
+        """
+        account = self.hook.get_connection(self.conn_id).extra_dejson.get("account")
+        return f"{self.sql_type}://{account}"
+
+    def truncate_table(self, table):
+        """Truncate table"""
+        self.run_sql(f"TRUNCATE {self.get_table_qualified_name(table)}")
+
 
 def wrap_identifier(inp: str) -> str:
     return f"Identifier(:{inp})"
