@@ -38,11 +38,32 @@ class S3Location(BaseFileLocation):
         bucket_name = url.netloc
         prefix = url.path[1:]
         prefixes = self.hook.list_keys(bucket_name=bucket_name, prefix=prefix)
-        paths = [
-            urlunparse((url.scheme, url.netloc, keys, "", "", "")) for keys in prefixes
-        ]
+        paths = [urlunparse((url.scheme, url.netloc, keys, "", "", "")) for keys in prefixes]
         return paths
 
     @property
     def size(self) -> int:
-        return -1
+        """Return file size for S3 location"""
+        url = urlparse(self.path)
+        bucket_name = url.netloc
+        object_name = url.path
+        if object_name.startswith("/"):
+            object_name = object_name[1:]
+        return self.hook.head_object(key=object_name, bucket_name=bucket_name).get("ContentLength") or -1
+
+    @property
+    def openlineage_dataset_namespace(self) -> str:
+        """
+        Returns the open lineage dataset namespace as per
+        https://github.com/OpenLineage/OpenLineage/blob/main/spec/Naming.md
+        """
+        parsed_url = urlparse(self.path)
+        return f"{parsed_url.scheme}://{parsed_url.netloc}"
+
+    @property
+    def openlineage_dataset_name(self) -> str:
+        """
+        Returns the open lineage dataset name as per
+        https://github.com/OpenLineage/OpenLineage/blob/main/spec/Naming.md
+        """
+        return urlparse(self.path).path
