@@ -1,6 +1,7 @@
+from __future__ import annotations
+
 import io
 from abc import ABC, abstractmethod
-from typing import Optional
 
 import pandas as pd
 
@@ -8,7 +9,7 @@ import pandas as pd
 class FileType(ABC):
     """Abstract File type class, meant to be the interface to all client code for all supported file types"""
 
-    def __init__(self, path: str, normalize_config: Optional[dict] = None):
+    def __init__(self, path: str, normalize_config: dict | None = None):
         self.path = path
         self.normalize_config = normalize_config
 
@@ -41,3 +42,6 @@ class FileType(ABC):
 
     def __repr__(self) -> str:
         return f'{self.__class__.__name__}(path="{self.path}")'
+
+    def __eq__(self, other):
+        return self.name == other.name
