@@ -1,22 +1,38 @@
 from __future__ import annotations
 
 import inspect
-from typing import Any
+from typing import Any, Sequence, cast
 
 import pandas as pd
 from airflow.decorators.base import DecoratedOperator
 from airflow.exceptions import AirflowException
+from openlineage.client.facet import (
+    BaseFacet,
+    DataQualityMetricsInputDatasetFacet,
+    DataSourceDatasetFacet,
+    OutputStatisticsOutputDatasetFacet,
+    SchemaDatasetFacet,
+    SchemaField,
+    SqlJobFacet,
+)
+from openlineage.client.run import Dataset as OpenlineageDataset
 from sqlalchemy.sql.functions import Function
 
+from astro.airflow.datasets import kwargs_with_datasets
 from astro.databases import create_database
 from astro.databases.base import BaseDatabase
-from astro.sql.table import Table
+from astro.lineage.extractor import OpenLineageFacets
+from astro.sql.operators.upstream_task_mixin import UpstreamTaskMixin
+from astro.table import BaseTable, Table
 from astro.utils.table import find_first_table
+from astro.utils.typing_compat import Context
 
 
-class BaseSQLDecoratedOperator(DecoratedOperator):
+class BaseSQLDecoratedOperator(UpstreamTaskMixin, DecoratedOperator):
     """Handles all decorator classes that can return a SQL function"""
 
+    template_fields: Sequence[str] = ("parameters", "op_args", "op_kwargs")
+
     database_impl: BaseDatabase
 
     def __init__(
@@ -26,29 +42,41 @@ class BaseSQLDecoratedOperator(DecoratedOperator):
         handler: Function | None = None,
         database: str | None = None,
         schema: str | None = None,
+        response_limit: int = -1,
+        response_size: int = -1,
         sql: str = "",
         **kwargs: Any,
     ):
         self.kwargs = kwargs or {}
         self.op_kwargs: dict = self.kwargs.get("op_kwargs") or {}
-        self.output_table: Table = self.op_kwargs.pop("output_table", Table())
+        self.output_table: BaseTable = self.op_kwargs.pop("output_table", Table())
         self.handler = self.op_kwargs.pop("handler", handler)
         self.conn_id = self.op_kwargs.pop("conn_id", conn_id)
+
         self.sql = sql
         self.parameters = parameters or {}
         self.database = self.op_kwargs.pop("database", database)
         self.schema = self.op_kwargs.pop("schema", schema)
+        self.response_limit = self.op_kwargs.pop("response_limit", response_limit)
+        self.response_size = self.op_kwargs.pop("response_size", response_size)
+
         self.op_args: dict[str, Table | pd.DataFrame] = {}
+
+        # We purposely do NOT render upstream_tasks otherwise we could have a case where a user
+        # has 10 dataframes as upstream tasks and it crashes the worker
+        upstream_tasks = self.op_kwargs.pop("upstream_tasks", [])
         super().__init__(
-            **kwargs,
+            upstream_tasks=upstream_tasks,
+            **kwargs_with_datasets(kwargs=kwargs, output_datasets=self.output_table),
         )
 
-    def execute(self, context: dict) -> None:
+    def execute(self, context: Context) -> None:
         first_table = find_first_table(
             op_args=self.op_args,  # type: ignore
             op_kwargs=self.op_kwargs,
             python_callable=self.python_callable,
             parameters=self.parameters,  # type: ignore
+            context=context,
         )
         if first_table:
             self.conn_id = self.conn_id or first_table.conn_id  # type: ignore
@@ -80,19 +108,20 @@ class BaseSQLDecoratedOperator(DecoratedOperator):
         self.read_sql_from_function()
         self.move_function_params_into_sql_params(context)
         self.translate_jinja_to_sqlalchemy_template(context)
+
         # if there is no SQL to run we raise an error
         if self.sql == "" or not self.sql:
             raise AirflowException("There's no SQL to run")
 
+        context["ti"].xcom_push(key="base_sql_query", value=str(self.sql))
+
     def create_output_table_if_needed(self) -> None:
         """
         If the user has not supplied an output table, this function creates one from scratch, otherwise populates
         the output table with necessary metadata.
         """
         self.output_table.conn_id = self.output_table.conn_id or self.conn_id
-        self.output_table = self.database_impl.populate_table_metadata(
-            self.output_table
-        )
+        self.output_table = self.database_impl.populate_table_metadata(self.output_table)
         self.log.info("Returning table %s", self.output_table)
 
     def read_sql_from_function(self) -> None:
@@ -152,13 +181,11 @@ class BaseSQLDecoratedOperator(DecoratedOperator):
         """
         # convert Jinja templating to SQLAlchemy SQL templating, safely converting table identifiers
         for k, v in self.parameters.items():
-            if isinstance(v, Table):
+            if isinstance(v, BaseTable):
                 (
                     jinja_table_identifier,
                     jinja_table_parameter_value,
-                ) = self.database_impl.get_sqlalchemy_template_table_identifier_and_parameter(
-                    v, k
-                )
+                ) = self.database_impl.get_sqlalchemy_template_table_identifier_and_parameter(v, k)
                 context[k] = jinja_table_identifier
                 self.parameters[k] = jinja_table_parameter_value
             else:
@@ -168,10 +195,58 @@ class BaseSQLDecoratedOperator(DecoratedOperator):
         if context:
             self.sql = self.render_template(self.sql, context)
 
+    def get_openlineage_facets(self, task_instance) -> OpenLineageFacets:
+        """
+        Returns the lineage data
+        """
+        input_uri = (
+            f"{self.output_table.openlineage_dataset_namespace()}"
+            f"://{self.output_table.openlineage_dataset_name()}"
+        )
+        input_dataset: list[OpenlineageDataset] = [
+            OpenlineageDataset(
+                namespace=self.output_table.openlineage_dataset_namespace(),
+                name=self.output_table.openlineage_dataset_name(),
+                facets={
+                    "schema": SchemaDatasetFacet(fields=[SchemaField(name=self.schema, type=self.database)]),
+                    "dataSource": DataSourceDatasetFacet(name=self.output_table.name, uri=input_uri),
+                },
+            )
+        ]
 
-def load_op_arg_dataframes_into_sql(
-    conn_id: str, op_args: tuple, target_table: Table
-) -> tuple:
+        output_uri = (
+            f"{self.output_table.openlineage_dataset_namespace()}"
+            f"://{self.output_table.openlineage_dataset_name()}"
+        )
+        output_dataset: list[OpenlineageDataset] = [OpenlineageDataset(namespace=None, name=None, facets={})]
+        if self.output_table:
+            output_dataset = [
+                OpenlineageDataset(
+                    namespace=self.output_table.openlineage_dataset_namespace(),
+                    name=self.output_table.openlineage_dataset_name(),
+                    facets={
+                        "outputStatistics": OutputStatisticsOutputDatasetFacet(
+                            rowCount=self.output_table.row_count
+                        ),
+                        "dataSource": DataSourceDatasetFacet(name=self.output_table.name, uri=output_uri),
+                        "dataQualityMetrics": DataQualityMetricsInputDatasetFacet(
+                            rowCount=self.output_table.row_count, columnMetrics={}
+                        ),
+                    },
+                )
+            ]
+
+        run_facets: dict[str, BaseFacet] = {}
+
+        base_sql_query = task_instance.xcom_pull(task_ids=task_instance.task_id, key="base_sql_query")
+        job_facets: dict[str, BaseFacet] = {"sql": SqlJobFacet(query=base_sql_query)}
+
+        return OpenLineageFacets(
+            inputs=input_dataset, outputs=output_dataset, run_facets=run_facets, job_facets=job_facets
+        )
+
+
+def load_op_arg_dataframes_into_sql(conn_id: str, op_args: tuple, target_table: BaseTable) -> tuple:
     """
     Identify dataframes in op_args and load them to the table.
 
@@ -184,11 +259,9 @@ def load_op_arg_dataframes_into_sql(
     database = create_database(conn_id=conn_id)
     for arg in op_args:
         if isinstance(arg, pd.DataFrame):
-            database.load_pandas_dataframe_to_table(
-                source_dataframe=arg, target_table=target_table
-            )
+            database.load_pandas_dataframe_to_table(source_dataframe=arg, target_table=target_table)
             final_args.append(target_table)
-        elif isinstance(arg, Table):
+        elif isinstance(arg, BaseTable):
             arg = database.populate_table_metadata(arg)
             final_args.append(arg)
         else:
@@ -196,9 +269,7 @@ def load_op_arg_dataframes_into_sql(
     return tuple(final_args)
 
 
-def load_op_kwarg_dataframes_into_sql(
-    conn_id: str, op_kwargs: dict, target_table: Table
-) -> dict:
+def load_op_kwarg_dataframes_into_sql(conn_id: str, op_kwargs: dict, target_table: BaseTable) -> dict:
     """
     Identify dataframes in op_kwargs and load them to a table.
 
@@ -211,12 +282,10 @@ def load_op_kwarg_dataframes_into_sql(
     database = create_database(conn_id=conn_id)
     for key, value in op_kwargs.items():
         if isinstance(value, pd.DataFrame):
-            df_table = target_table.create_similar_table()
-            database.load_pandas_dataframe_to_table(
-                source_dataframe=value, target_table=df_table
-            )
+            df_table = cast(BaseTable, target_table.create_similar_table())
+            database.load_pandas_dataframe_to_table(source_dataframe=value, target_table=df_table)
             final_kwargs[key] = df_table
-        elif isinstance(value, Table):
+        elif isinstance(value, BaseTable):
             value = database.populate_table_metadata(value)
             final_kwargs[key] = value
         else:
