@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import logging
 from collections.abc import Iterable, Mapping
 from typing import Any, Callable
 
@@ -9,7 +10,10 @@ except ImportError:
     from airflow.decorators.base import task_decorator_factory
     from airflow.decorators import _TaskDecorator as TaskDecorator
 
+from astro import settings
+from astro.exceptions import IllegalLoadToDatabaseException
 from astro.sql.operators.base_decorator import BaseSQLDecoratedOperator
+from astro.utils.typing_compat import Context
 
 
 class RawSQLOperator(BaseSQLDecoratedOperator):
@@ -21,14 +25,26 @@ class RawSQLOperator(BaseSQLDecoratedOperator):
     and on the SQL statement/function declared by the user.
     """
 
-    def execute(self, context: dict) -> Any:
+    def execute(self, context: Context) -> Any:
         super().execute(context)
 
-        result = self.database_impl.run_sql(
-            sql_statement=self.sql, parameters=self.parameters
-        )
+        result = self.database_impl.run_sql(sql=self.sql, parameters=self.parameters)
+        if self.response_size == -1 and not settings.IS_CUSTOM_XCOM_BACKEND:
+            logging.warning(
+                "Using `run_raw_sql` without `response_size` can result in excessive amount of data being recorded "
+                "to the Airflow metadata database, leading to issues to the orchestration of tasks. It is possible to "
+                "avoid this problem by either setting `response_size` to a small integer or by using a custom XCom "
+                "backend."
+            )
+
         if self.handler:
-            return self.handler(result)
+            response = self.handler(result)
+            if self.response_limit >= 0 and len(response) > self.response_limit:
+                raise IllegalLoadToDatabaseException()  # pragma: no cover
+            if self.response_size >= 0:
+                return response[: self.response_size]
+            else:
+                return response
         else:
             return None
 
@@ -40,6 +56,7 @@ def run_raw_sql(
     database: str | None = None,
     schema: str | None = None,
     handler: Callable | None = None,
+    response_size: int = settings.RAW_SQL_MAX_RESPONSE_SIZE,
     **kwargs: Any,
 ) -> TaskDecorator:
     """
@@ -51,7 +68,7 @@ def run_raw_sql(
 
     .. code-block:: python
 
-      @transform
+      @run_raw_sql
       def my_sql_statement(table1: Table) -> Table:
           return "DROP TABLE {{table1}}"
 
@@ -63,20 +80,26 @@ def run_raw_sql(
     result of a provided ``handler`` function or it will not return anything at all.
 
 
-    :param python_callable: This parameter is filled in automatically when you use the transform function as a decorator
-        This is where the python function gets passed to the wrapping function
-    :param conn_id: Connection ID for the database you want to connect to. If you do not pass in a value for this object
-        we can infer the connection ID from the first table passed into the python_callable function.
-        (required if there are no table arguments)
+    :param python_callable: This parameter is filled in automatically when you use the transform function as a
+        decorator. This is where the python function gets passed to the wrapping function
+    :param conn_id: Connection ID for the database you want to connect to.
+        If you do not pass in a value for this object we can infer the connection ID from the first table
+        passed into the python_callable function. (required if there are no table arguments)
     :param parameters: parameters to pass into the SQL query
     :param database: Database within the SQL instance you want to access. If left blank we will default to the
-        table.metatadata.database in the first Table passed to the function (required if there are no table arguments)
+        table.metatadata.database in the first Table passed to the function
+        (required if there are no table arguments)
     :param schema: Schema within the SQL instance you want to access. If left blank we will default to the
-        table.metatadata.schema in the first Table passed to the function (required if there are no table arguments)
+        table.metatadata.schema in the first Table passed to the function
+        (required if there are no table arguments)
     :param handler: Handler function to process the result of the SQL query. For more information please consult
         https://docs.sqlalchemy.org/en/14/core/connections.html#sqlalchemy.engine.Result
+    :param response_size: Used to trim the responses returned to avoid trashing the Airflow DB.
+        The default value is -1, which means the response is not changed. Otherwise, if the response is a list,
+        returns up to the desired amount of items. If the response is a string, trims it to the desired size.
     :param kwargs:
-    :return: By default returns None unless there is a handler function, in which case returns the result of the handler
+    :return: By default returns None unless there is a handler function,
+        in which case returns the result of the handler
     """
 
     kwargs.update(
@@ -86,6 +109,7 @@ def run_raw_sql(
             "database": database,
             "schema": schema,
             "handler": handler,
+            "response_size": response_size,
         }
     )
     return task_decorator_factory(
